"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CustomMappingInputTypedDict(TypedDict):
    value: NotRequired[str]
    r"""Target Field Mapping value"""


class CustomMappingInput(BaseModel):
    value: Optional[str] = None
    r"""Target Field Mapping value"""
