"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .formfieldoptiongroup import FormFieldOptionGroup, FormFieldOptionGroupTypedDict
from .simpleformfieldoption import SimpleFormFieldOption, SimpleFormFieldOptionTypedDict
from typing import Union
from typing_extensions import TypeAliasType


FormFieldOptionTypedDict = TypeAliasType(
    "FormFieldOptionTypedDict",
    Union[SimpleFormFieldOptionTypedDict, FormFieldOptionGroupTypedDict],
)


FormFieldOption = TypeAliasType(
    "FormFieldOption", Union[SimpleFormFieldOption, FormFieldOptionGroup]
)
