"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class WebhookEventType(str, Enum):
    WILDCARD_ = "*"
    CRM_ACTIVITY_CREATED = "crm.activity.created"
    CRM_ACTIVITY_UPDATED = "crm.activity.updated"
    CRM_ACTIVITY_DELETED = "crm.activity.deleted"
    CRM_COMPANY_CREATED = "crm.company.created"
    CRM_COMPANY_UPDATED = "crm.company.updated"
    CRM_COMPANY_DELETED = "crm.company.deleted"
    CRM_CONTACT_CREATED = "crm.contact.created"
    CRM_CONTACT_UPDATED = "crm.contact.updated"
    CRM_CONTACT_DELETED = "crm.contact.deleted"
    CRM_LEAD_CREATED = "crm.lead.created"
    CRM_LEAD_UPDATED = "crm.lead.updated"
    CRM_LEAD_DELETED = "crm.lead.deleted"
    CRM_NOTE_CREATED = "crm.note.created"
    CRM_NOTES_UPDATED = "crm.notes.updated"
    CRM_NOTES_DELETED = "crm.notes.deleted"
    CRM_OPPORTUNITY_CREATED = "crm.opportunity.created"
    CRM_OPPORTUNITY_UPDATED = "crm.opportunity.updated"
    CRM_OPPORTUNITY_DELETED = "crm.opportunity.deleted"
    LEAD_LEAD_CREATED = "lead.lead.created"
    LEAD_LEAD_UPDATED = "lead.lead.updated"
    LEAD_LEAD_DELETED = "lead.lead.deleted"
    VAULT_CONNECTION_CREATED = "vault.connection.created"
    VAULT_CONNECTION_UPDATED = "vault.connection.updated"
    VAULT_CONNECTION_DISABLED = "vault.connection.disabled"
    VAULT_CONNECTION_DELETED = "vault.connection.deleted"
    VAULT_CONNECTION_CALLABLE = "vault.connection.callable"
    VAULT_CONNECTION_REVOKED = "vault.connection.revoked"
    VAULT_CONNECTION_TOKEN_REFRESH_FAILED = "vault.connection.token_refresh.failed"
    ATS_JOB_CREATED = "ats.job.created"
    ATS_JOB_UPDATED = "ats.job.updated"
    ATS_JOB_DELETED = "ats.job.deleted"
    ATS_APPLICANT_CREATED = "ats.applicant.created"
    ATS_APPLICANT_UPDATED = "ats.applicant.updated"
    ATS_APPLICANT_DELETED = "ats.applicant.deleted"
    ACCOUNTING_CUSTOMER_CREATED = "accounting.customer.created"
    ACCOUNTING_CUSTOMER_UPDATED = "accounting.customer.updated"
    ACCOUNTING_CUSTOMER_DELETED = "accounting.customer.deleted"
    ACCOUNTING_INVOICE_CREATED = "accounting.invoice.created"
    ACCOUNTING_INVOICE_UPDATED = "accounting.invoice.updated"
    ACCOUNTING_INVOICE_DELETED = "accounting.invoice.deleted"
    ACCOUNTING_INVOICE_ITEM_CREATED = "accounting.invoice_item.created"
    ACCOUNTING_INVOICE_ITEM_UPDATED = "accounting.invoice_item.updated"
    ACCOUNTING_INVOICE_ITEM_DELETED = "accounting.invoice_item.deleted"
    ACCOUNTING_LEDGER_ACCOUNT_CREATED = "accounting.ledger_account.created"
    ACCOUNTING_LEDGER_ACCOUNT_UPDATED = "accounting.ledger_account.updated"
    ACCOUNTING_LEDGER_ACCOUNT_DELETED = "accounting.ledger_account.deleted"
    ACCOUNTING_TAX_RATE_CREATED = "accounting.tax_rate.created"
    ACCOUNTING_TAX_RATE_UPDATED = "accounting.tax_rate.updated"
    ACCOUNTING_TAX_RATE_DELETED = "accounting.tax_rate.deleted"
    ACCOUNTING_BILL_CREATED = "accounting.bill.created"
    ACCOUNTING_BILL_UPDATED = "accounting.bill.updated"
    ACCOUNTING_BILL_DELETED = "accounting.bill.deleted"
    ACCOUNTING_PAYMENT_CREATED = "accounting.payment.created"
    ACCOUNTING_PAYMENT_UPDATED = "accounting.payment.updated"
    ACCOUNTING_PAYMENT_DELETED = "accounting.payment.deleted"
    ACCOUNTING_SUPPLIER_CREATED = "accounting.supplier.created"
    ACCOUNTING_SUPPLIER_UPDATED = "accounting.supplier.updated"
    ACCOUNTING_SUPPLIER_DELETED = "accounting.supplier.deleted"
    ACCOUNTING_PURCHASE_ORDER_CREATED = "accounting.purchase_order.created"
    ACCOUNTING_PURCHASE_ORDER_UPDATED = "accounting.purchase_order.updated"
    ACCOUNTING_PURCHASE_ORDER_DELETED = "accounting.purchase_order.deleted"
    POS_ORDER_CREATED = "pos.order.created"
    POS_ORDER_UPDATED = "pos.order.updated"
    POS_ORDER_DELETED = "pos.order.deleted"
    POS_PRODUCT_CREATED = "pos.product.created"
    POS_PRODUCT_UPDATED = "pos.product.updated"
    POS_PRODUCT_DELETED = "pos.product.deleted"
    POS_PAYMENT_CREATED = "pos.payment.created"
    POS_PAYMENT_UPDATED = "pos.payment.updated"
    POS_PAYMENT_DELETED = "pos.payment.deleted"
    POS_MERCHANT_CREATED = "pos.merchant.created"
    POS_MERCHANT_UPDATED = "pos.merchant.updated"
    POS_MERCHANT_DELETED = "pos.merchant.deleted"
    POS_LOCATION_CREATED = "pos.location.created"
    POS_LOCATION_UPDATED = "pos.location.updated"
    POS_LOCATION_DELETED = "pos.location.deleted"
    POS_ITEM_CREATED = "pos.item.created"
    POS_ITEM_UPDATED = "pos.item.updated"
    POS_ITEM_DELETED = "pos.item.deleted"
    POS_MODIFIER_CREATED = "pos.modifier.created"
    POS_MODIFIER_UPDATED = "pos.modifier.updated"
    POS_MODIFIER_DELETED = "pos.modifier.deleted"
    POS_MODIFIER_GROUP_CREATED = "pos.modifier-group.created"
    POS_MODIFIER_GROUP_UPDATED = "pos.modifier-group.updated"
    POS_MODIFIER_GROUP_DELETED = "pos.modifier-group.deleted"
    HRIS_EMPLOYEE_CREATED = "hris.employee.created"
    HRIS_EMPLOYEE_UPDATED = "hris.employee.updated"
    HRIS_EMPLOYEE_DELETED = "hris.employee.deleted"
    HRIS_EMPLOYEE_TERMINATED = "hris.employee.terminated"
    HRIS_COMPANY_CREATED = "hris.company.created"
    HRIS_COMPANY_UPDATED = "hris.company.updated"
    HRIS_COMPANY_DELETED = "hris.company.deleted"
    FILE_STORAGE_FILE_CREATED = "file-storage.file.created"
    FILE_STORAGE_FILE_UPDATED = "file-storage.file.updated"
    FILE_STORAGE_FILE_DELETED = "file-storage.file.deleted"
    ISSUE_TRACKING_TICKET_CREATED = "issue-tracking.ticket.created"
    ISSUE_TRACKING_TICKET_UPDATED = "issue-tracking.ticket.updated"
    ISSUE_TRACKING_TICKET_DELETED = "issue-tracking.ticket.deleted"
    ATS_APPLICATION_CREATED = "ats.application.created"
    ATS_APPLICATION_UPDATED = "ats.application.updated"
    ATS_APPLICATION_DELETED = "ats.application.deleted"
    ACCOUNTING_EXPENSE_CREATED = "accounting.expense.created"
    ACCOUNTING_EXPENSE_UPDATED = "accounting.expense.updated"
    ACCOUNTING_EXPENSE_DELETED = "accounting.expense.deleted"
