"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class IntegrationState(str, Enum):
    r"""The current state of the Integration."""

    DISABLED = "disabled"
    NEEDS_CONFIGURATION = "needs_configuration"
    CONFIGURED = "configured"
