"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .deletewebhookresponse import DeleteWebhookResponse, DeleteWebhookResponseTypedDict
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from .unexpectederrorresponse import (
    UnexpectedErrorResponse,
    UnexpectedErrorResponseTypedDict,
)
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata, HeaderMetadata, PathParamMetadata
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WebhookWebhooksDeleteGlobalsTypedDict(TypedDict):
    app_id: NotRequired[str]
    r"""The ID of your Unify application"""


class WebhookWebhooksDeleteGlobals(BaseModel):
    app_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-app-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of your Unify application"""


class WebhookWebhooksDeleteRequestTypedDict(TypedDict):
    id: str
    r"""JWT Webhook token that represents the unifiedApi and applicationId associated to the event source."""


class WebhookWebhooksDeleteRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""JWT Webhook token that represents the unifiedApi and applicationId associated to the event source."""


class WebhookWebhooksDeleteResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    delete_webhook_response: NotRequired[DeleteWebhookResponseTypedDict]
    r"""Webhooks"""
    unexpected_error_response: NotRequired[UnexpectedErrorResponseTypedDict]
    r"""Unexpected error"""


class WebhookWebhooksDeleteResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    delete_webhook_response: Optional[DeleteWebhookResponse] = None
    r"""Webhooks"""

    unexpected_error_response: Optional[UnexpectedErrorResponse] = None
    r"""Unexpected error"""
