"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class UnifiedAPIID(str, Enum):
    r"""Name of Apideck Unified API"""

    ACCOUNTING = "accounting"
    ATS = "ats"
    CALENDAR = "calendar"
    CRM = "crm"
    CSP = "csp"
    CUSTOMER_SUPPORT = "customer-support"
    ECOMMERCE = "ecommerce"
    EMAIL = "email"
    EMAIL_MARKETING = "email-marketing"
    EXPENSE_MANAGEMENT = "expense-management"
    FILE_STORAGE = "file-storage"
    FORM = "form"
    HRIS = "hris"
    LEAD = "lead"
    PAYROLL = "payroll"
    POS = "pos"
    PROCUREMENT = "procurement"
    PROJECT_MANAGEMENT = "project-management"
    SCRIPT = "script"
    SMS = "sms"
    SPREADSHEET = "spreadsheet"
    TEAM_MESSAGING = "team-messaging"
    ISSUE_TRACKING = "issue-tracking"
    TIME_REGISTRATION = "time-registration"
    TRANSACTIONAL_EMAIL = "transactional-email"
    VAULT = "vault"
    DATA_WAREHOUSE = "data-warehouse"
