"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .simpleformfieldoption import SimpleFormFieldOption, SimpleFormFieldOptionTypedDict
from apideck_unify.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class FormFieldOptionGroupTypedDict(TypedDict):
    id: NotRequired[str]
    label: NotRequired[str]
    options: NotRequired[List[SimpleFormFieldOptionTypedDict]]


class FormFieldOptionGroup(BaseModel):
    id: Optional[str] = None

    label: Optional[str] = None

    options: Optional[List[SimpleFormFieldOption]] = None
