"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class LinkedParentCustomerTypedDict(TypedDict):
    r"""The parent customer this entity is linked to."""

    id: str
    r"""The parent ID of the customer this entity is linked to."""
    name: NotRequired[str]
    r"""The name of the parent customer."""


class LinkedParentCustomer(BaseModel):
    r"""The parent customer this entity is linked to."""

    id: str
    r"""The parent ID of the customer this entity is linked to."""

    name: Optional[str] = None
    r"""The name of the parent customer."""
