"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .connectorstatus import ConnectorStatus
from .unifiedapiid import UnifiedAPIID
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ConnectorsFilterTypedDict(TypedDict):
    unified_api: NotRequired[UnifiedAPIID]
    r"""Name of Apideck Unified API"""
    status: NotRequired[ConnectorStatus]
    r"""Status of the connector. Connectors with status live or beta are callable."""


class ConnectorsFilter(BaseModel):
    unified_api: Annotated[Optional[UnifiedAPIID], FieldMetadata(query=True)] = None
    r"""Name of Apideck Unified API"""

    status: Annotated[Optional[ConnectorStatus], FieldMetadata(query=True)] = None
    r"""Status of the connector. Connectors with status live or beta are callable."""
